/*!
FullCalendar ScrollGrid Plugin v7.0.0-beta.0
Docs & License: https://fullcalendar.io/docs/premium
(c) 2024 Adam Shaw
*/
FullCalendar.ScrollGrid = (function (exports, core, premiumCommonPlugin, internal$1) {
    'use strict';

    function _interopDefault (e) { return e && e.__esModule ? e : { 'default': e }; }

    var premiumCommonPlugin__default = /*#__PURE__*/_interopDefault(premiumCommonPlugin);

    const WHEEL_EVENT_NAMES = 'wheel mousewheel DomMouseScroll MozMousePixelScroll'.split(' ');
    /*
    ALSO, with the ability to disable touch
    */
    class ScrollListener {
        constructor(el) {
            this.el = el;
            this.emitter = new internal$1.Emitter();
            this.isScrolling = false;
            this.isTouching = false; // user currently has finger down?
            this.isRecentlyWheeled = false;
            this.isRecentlyScrolled = false;
            this.wheelWaiter = new internal$1.DelayedRunner(this._handleWheelWaited.bind(this));
            this.scrollWaiter = new internal$1.DelayedRunner(this._handleScrollWaited.bind(this));
            // Handlers
            // ----------------------------------------------------------------------------------------------
            this.handleScroll = () => {
                this.startScroll();
                this.emitter.trigger('scroll', this.isRecentlyWheeled, this.isTouching);
                this.isRecentlyScrolled = true;
                this.scrollWaiter.request(500);
            };
            // will fire *before* the scroll event is fired (might not cause a scroll)
            this.handleWheel = () => {
                this.isRecentlyWheeled = true;
                this.wheelWaiter.request(500);
            };
            // will fire *before* the scroll event is fired (might not cause a scroll)
            this.handleTouchStart = () => {
                this.isTouching = true;
            };
            this.handleTouchEnd = () => {
                this.isTouching = false;
                // if the user ended their touch, and the scroll area wasn't moving,
                // we consider this to be the end of the scroll.
                if (!this.isRecentlyScrolled) {
                    this.endScroll(); // won't fire if already ended
                }
            };
            el.addEventListener('scroll', this.handleScroll);
            el.addEventListener('touchstart', this.handleTouchStart, { passive: true });
            el.addEventListener('touchend', this.handleTouchEnd);
            for (let eventName of WHEEL_EVENT_NAMES) {
                el.addEventListener(eventName, this.handleWheel, { passive: true });
            }
        }
        destroy() {
            let { el } = this;
            el.removeEventListener('scroll', this.handleScroll);
            el.removeEventListener('touchstart', this.handleTouchStart, { passive: true });
            el.removeEventListener('touchend', this.handleTouchEnd);
            for (let eventName of WHEEL_EVENT_NAMES) {
                el.removeEventListener(eventName, this.handleWheel, { passive: true });
            }
        }
        // Start / Stop
        // ----------------------------------------------------------------------------------------------
        startScroll() {
            if (!this.isScrolling) {
                this.isScrolling = true;
                this.emitter.trigger('scrollStart', this.isRecentlyWheeled, this.isTouching);
            }
        }
        endScroll() {
            if (this.isScrolling) {
                this.emitter.trigger('scrollEnd');
                this.isScrolling = false;
                this.isRecentlyScrolled = true;
                this.isRecentlyWheeled = false;
                this.scrollWaiter.clear();
                this.wheelWaiter.clear();
            }
        }
        _handleScrollWaited() {
            this.isRecentlyScrolled = false;
            // only end the scroll if not currently touching.
            // if touching, the scrolling will end later, on touchend.
            if (!this.isTouching) {
                this.endScroll(); // won't fire if already ended
            }
        }
        _handleWheelWaited() {
            this.isRecentlyWheeled = false;
        }
    }

    class ScrollerSyncer {
        constructor(isHorizontal = false) {
            this.isHorizontal = isHorizontal;
            this.scrollers = []; // TODO: move away from requiring Scroller
            this.scrollListeners = [];
            this.isPaused = false;
        }
        handleChildren(scrollers, isRtl) {
            if (!internal$1.isArraysEqual(this.scrollers, scrollers)) {
                this.destroy();
                this.scrollers = scrollers;
                const scrollListeners = [];
                for (const scroller of scrollers) {
                    if (scroller) { // could be null
                        scrollListeners.push(this.bindScroller(scroller.el));
                    }
                }
                this.scrollListeners = scrollListeners;
            }
            this.isRtl = isRtl;
        }
        destroy() {
            for (let scrollListener of this.scrollListeners) {
                scrollListener.destroy();
            }
        }
        get x() {
            const { scrollListeners, masterEl, isRtl } = this;
            const el = masterEl || (scrollListeners.length ? scrollListeners[0].el : undefined);
            return internal$1.getNormalizedScrollX(el, isRtl);
        }
        get y() {
            const { scrollListeners, masterEl } = this;
            const el = masterEl || (scrollListeners.length ? scrollListeners[0].el : undefined);
            return el.scrollTop;
        }
        scrollTo({ x, y }) {
            this.isPaused = true;
            const { scrollListeners, isRtl } = this;
            if (y != null) {
                for (let scrollListener of scrollListeners) {
                    scrollListener.el.scrollTop = y;
                }
            }
            if (x != null) {
                for (let scrollListener of scrollListeners) {
                    internal$1.setNormalizedScrollX(scrollListener.el, isRtl, x);
                }
            }
            this.isPaused = false;
        }
        bindScroller(el) {
            let { isHorizontal } = this;
            let scrollListener = new ScrollListener(el);
            const onScroll = (isWheel, isTouch) => {
                if (!this.isPaused) {
                    if (!this.masterEl || (this.masterEl !== el && (isWheel || isTouch))) {
                        this.assignMaster(el);
                    }
                    if (this.masterEl === el) { // dealing with current
                        for (let scrollListener of this.scrollListeners) {
                            const otherEl = scrollListener.el;
                            if (otherEl !== el) {
                                if (isHorizontal) {
                                    otherEl.scrollLeft = el.scrollLeft;
                                }
                                else {
                                    otherEl.scrollTop = el.scrollTop;
                                }
                            }
                        }
                    }
                }
            };
            const onScrollEnd = () => {
                if (this.masterEl === el) {
                    this.masterEl = null;
                }
            };
            scrollListener.emitter.on('scroll', onScroll);
            scrollListener.emitter.on('scrollEnd', onScrollEnd);
            return scrollListener;
        }
        assignMaster(el) {
            this.masterEl = el;
            for (let scrollListener of this.scrollListeners) {
                if (scrollListener.el !== el) {
                    scrollListener.endScroll(); // to prevent residual scrolls from reclaiming master
                }
            }
        }
    }

    var plugin = core.createPlugin({
        name: '@fullcalendar/scrollgrid',
        premiumReleaseDate: '2024-10-01',
        deps: [premiumCommonPlugin__default["default"]],
        scrollerSyncerClass: ScrollerSyncer
    });

    // new

    var internal = {
        __proto__: null,
        ScrollerSyncer: ScrollerSyncer
    };

    core.globalPlugins.push(plugin);

    exports.Internal = internal;
    exports["default"] = plugin;

    Object.defineProperty(exports, '__esModule', { value: true });

    return exports;

})({}, FullCalendar, FullCalendar.PremiumCommon, FullCalendar.Internal);
